/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Jan 27, 2008
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.uvos.webapp.actions.appform;

import java.util.HashMap;
import java.util.Map;

import org.apache.xmlbeans.XmlException;
import org.springframework.validation.Errors;
import org.springframework.webflow.action.FormAction;
import org.springframework.webflow.core.collection.MutableAttributeMap;
import org.springframework.webflow.execution.Event;
import org.springframework.webflow.execution.RequestContext;

import pl.edu.icm.unicore.uvos.api.Attribute;
import pl.edu.icm.unicore.uvos.api.Group;
import pl.edu.icm.unicore.uvos.api.IdentityType;
import pl.edu.icm.unicore.uvos.api.VOApplicationForm;
import pl.edu.icm.unicore.uvos.api.exceptions.InternalException;
import pl.edu.icm.unicore.uvos.api.exceptions.SecurityException;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.XMLBeansMapper;
import pl.edu.icm.uvos.webapp.data.DataStore;
import xmlbeans.pl.edu.icm.uvos.internal.AttributeXType;
import xmlbeans.pl.edu.icm.uvos.internal.GroupXType;
import xmlbeans.pl.edu.icm.uvos.internal.app.VOApplicationFormConstraintsType;


/**
 * @author K. Benedyczak
 */
public class SelectedFormAction extends FormAction
{
	public static String[] DEFAULT_ID_TYPES = {IdentityType.DN.getName(),
		IdentityType.EMAIL.getName(), IdentityType.X509CERT.getName()}; 
	private DataStore dao;
	
	private static final String KEY_ALLOWED_FORMATS = "allowedIdFormats";
	private static final String KEY_AGREEMENT = "agreement";
	
	public SelectedFormAction(DataStore dao)
	{
		super();
		this.dao = dao;
	}

	public Event setupData(RequestContext context) 
		throws InternalException, SecurityException, XmlException
	{
		MutableAttributeMap req = context.getFlowScope();
		String idRaw = (String) req.get("formId");
		
		if (idRaw == null)
			throw new InternalException("Wrong (empty) form id passed.");
		VOApplicationForm form;
		try
		{
			int id = Integer.parseInt(idRaw);
			form = dao.getApplicationFormById(id);
		} catch (NumberFormatException e)
		{
			throw new InternalException("Wrong (not an integer) form id passed.");
		}
		
		
		Map<String, Object> selectedF = new HashMap<String, Object>();
		
		VOApplicationFormConstraintsType constraints;		
		try
		{
			constraints = VOApplicationFormConstraintsType.Factory.parse(
					form.getConstraints());
		} catch (XmlException e)
		{
			throw new InternalException("VO application form's " +
					"constraints are invalid");
		} 
		
		selectedF.put("isVO", new Boolean(form.getBaseGroup().getPath().length <= 1));
		selectedF.put("canDoCSR", new Boolean(constraints.getAllowCSR() != null));
		selectedF.put("name", form.getFriendlyName());
		selectedF.put("description", form.getDescription());
		selectedF.put(KEY_AGREEMENT, form.getAgreement());
		selectedF.put("baseGroup", form.getBaseGroup().toString());
		if (constraints.getDisableUserCASubmission() || 
				!constraints.isSetDisableUserCASubmission())
			selectedF.put("disableCustomCA", "true");
		else
			selectedF.put("disableCustomCA", "false");
		if (constraints.getDisableCertificateValidation() || 
				!constraints.isSetDisableUserCASubmission())
			req.put("disableCertificateCheck", true);
		else
			req.put("disableCertificateCheck", false);

		//those two are to overcome single checkbox problem
		Map<String, String> accepted = new HashMap<String, String>();
		accepted.put("acceptedId", "I do accept the above agreement");
		selectedF.put("accepted", accepted);

		Map<String, String> isCSR = new HashMap<String, String>();
		isCSR.put("isCSR", "I wish to submit a CSR");
		selectedF.put("isCSR", isCSR);
		
		
		String allowedFormats[] = constraints.getAllowedIdFormatsArray();
		Map<String, String> idF = new HashMap<String, String>();
		Map<String, String> idFN = new HashMap<String, String>();
		if (allowedFormats == null || allowedFormats.length == 0)
			allowedFormats = DEFAULT_ID_TYPES;
		for (int i=0; i<allowedFormats.length; i++)
		{
			idFN.put("" + i, allowedFormats[i].substring(
					allowedFormats[i].lastIndexOf(":") + 1));
			idF.put("" + i, allowedFormats[i]);
		}
		selectedF.put(KEY_ALLOWED_FORMATS, idF);
		selectedF.put("allowedIdFormatsNice", idFN);
		
		Map<String, String> adG = new HashMap<String, String>();
		GroupXType []groupsSelection = constraints.getSubgroupsSelectionArray();
		if (groupsSelection != null && groupsSelection.length > 0 && 
				!constraints.getForceGroupSelection())
		{
			for (int i=0; i<groupsSelection.length; i++)
			{
				Group g = XMLBeansMapper.map2APIGroup(groupsSelection[i]); 
				adG.put("" + i, g.toString());
			}
			selectedF.put("additionalGroups", adG);
		}

		Map<String, String> adA = new HashMap<String, String>();
		AttributeXType []attrsSelection = constraints.getAttributesSelectionArray();
		if (attrsSelection != null && attrsSelection.length > 0 &&
				!constraints.getForceAttributeSelection())
		{
			for (int i=0; i<attrsSelection.length; i++)
			{
				Attribute a = XMLBeansMapper.map2APIAttribute(
						attrsSelection[i]);
				String value = "";
				if (a.getValues().size() > 0)
				{
					for (String v: a.getValues())
						value += ", " + v;
					value = value.substring(2);
					value = " values: " + value;
				}
				adA.put("" + i, a.toString());
			}
			selectedF.put("additionalAttributes", adA);
		}
		
		req.put("form", selectedF);
		return success();
	}

	@SuppressWarnings("unchecked")
	@Override
	protected void doValidate(RequestContext context, Object formObject,
			Errors errors) throws java.lang.Exception
	{
		super.doValidate(context, formObject, errors);
		AppFormData dat = (AppFormData) formObject;
		MutableAttributeMap map = context.getFlowScope();
		HashMap<String, Object> formData = 
			(HashMap<String, Object>) map.get("form");
		Map<String, String> idFormats = 
			(Map<String, String>) formData.get(KEY_ALLOWED_FORMATS);
		String selectedFormat = idFormats.get(dat.getIdFormat());

		String agreement = (String)formData.get(KEY_AGREEMENT);
		if ((dat.getAccepted() == null || dat.getAccepted().size() == 0) &&
				agreement.trim().length() > 0)
			errors.rejectValue("accepted", 
					"accepted", "You must accept the terms of VO.");
		String email = dat.getEmail();
		if (email != null && email.length() > 0)
		{
			int mi = email.indexOf("@");
			if (mi == -1 || mi == 0 || mi == email.length() - 1)
				errors.rejectValue("email", "email", 
					"Your email address is incorrect");
		} else
		{
			if (!selectedFormat.equals(IdentityType.EMAIL.getName()))
				errors.rejectValue("email", "email", 
					"You must enter an email address");
		}
		if (selectedFormat.equals(IdentityType.EMAIL.getName()) &&
				(dat.getIsCSR() != null && dat.getIsCSR().size() > 0))
		{
			errors.rejectValue("isCSR", "isCSR", 
				"You must not ask for CSR when using" +
				" an email address as an identity format");
		}
			
	}

	@SuppressWarnings("unchecked")
	public Event postprocessData(RequestContext context) 
		throws InternalException, SecurityException, XmlException
	{
		MutableAttributeMap req = context.getFlowScope();
		Map<String, Object> app = new HashMap<String, Object>();
		AppFormData appData =  (AppFormData) req.get("app1FormData");
		String format = appData.getIdFormat();
		HashMap<String, Object> formData = (HashMap<String, Object>) req.get("form");
		Map<String, String> idFormats = (Map<String, String>) formData.get("allowedIdFormats");
		String selectedFormat = idFormats.get(format);
		Map<String, String> idFormatsNice = (Map<String, String>) formData.get("allowedIdFormatsNice");
		String selectedFormatNice = idFormatsNice.get(format);
		app.put("selectedFormat", selectedFormat);
		app.put("selectedFormatNice", selectedFormatNice);
		req.put("application", app);
		return success();
	}
}






