/*
 * Copyright (c) 2007, 2008 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on Jan 29, 2008
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.uvos.webapp.actions.idform;

import org.springframework.web.multipart.MultipartFile;
import org.springframework.webflow.action.AbstractAction;
import org.springframework.webflow.execution.Event;
import org.springframework.webflow.execution.RequestContext;

/**
 * @author K. Benedyczak
 */
public class IdentityUploadAction extends AbstractAction
{
	public static final int MAX_FILE = 20480;
	/**
	 * Here we read all files and put them or apropriate errors into 
	 * form backing object. This is needed as Validator (which is executed next)
	 * should know if and what files were provided.
	 */
	protected Event doExecute(RequestContext context) throws Exception
	{
		IdFormData backing = (IdFormData) context.getFlowScope().get("app2FormData");
		backing.setCaCert(null);
		backing.setCaCertError(null);
		backing.setIdCert(null);
		backing.setIdCertError(null);
		
		MultipartFile file = context.getRequestParameters()
				.getMultipartFile("caCert_file");
		
		if (file != null && file.getSize() > 0 && file.getSize() < MAX_FILE)
			backing.setCaCert(new String(file.getBytes()));
		else if (file != null && file.getSize() > 0)
			backing.setCaCertError("The uploaded file is too " +
					"large (maximum size is " + MAX_FILE + ")");
		
		file = context.getRequestParameters().getMultipartFile("idCert_file");
		if (file != null && file.getSize() > 0 && file.getSize() < MAX_FILE)
			backing.setIdCert(new String(file.getBytes()));
		else
		{
			if (file == null || file.getSize() == 0)
				backing.setIdCertError("The file was not uploaded");
			else
				backing.setIdCertError("The uploaded file is too " +
					"large (maximum size is " + MAX_FILE + ")");
		}
		return success();
	}
}








