/*
 * Copyright (c) 2010 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on 03-12-2010
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */
package pl.edu.icm.uvos.webapp.data;

import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.security.KeyStore;
import java.security.cert.TrustAnchor;
import java.security.cert.X509Certificate;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Set;

import org.apache.log4j.Logger;
import org.picocontainer.PicoContainer;

import pl.edu.icm.unicore.uvos.util.ConfDefaults;
import pl.edu.icm.unicore.uvos.util.Configuration;

public class TrustStoreImpl implements TrustStore
{
	private Set<TrustAnchor> taSet;
	private static final Logger log = Logger.getLogger(TrustStoreImpl.class);

	public TrustStoreImpl()
	{
		PicoContainer pico = PicoAccessor.getPico();
		if (pico == null) 
			return;

		Configuration conf = pico.getComponent(Configuration.class);
		String passwd = conf.getAsString(ConfDefaults.TRUSTSTORE_PASSWD, 
				ConfDefaults.truststorePasswd, null);
		String truststoreFile = conf.getAsString(ConfDefaults.TRUSTSTORE,
				ConfDefaults.truststore, null);
		String type = conf.getAsString(ConfDefaults.TRUSTSTORE_TYPE,
				ConfDefaults.truststoreType, null);
		try
		{
			KeyStore truststore = KeyStore.getInstance(type);
			truststore.load(new BufferedInputStream(new FileInputStream(truststoreFile)), 
					passwd.toCharArray());

			taSet = new HashSet<TrustAnchor>();
			Enumeration<String> aliases = truststore.aliases();
			while (aliases.hasMoreElements())
			{
				String alias = aliases.nextElement();
				if (truststore.isCertificateEntry(alias))
					taSet.add(new TrustAnchor((X509Certificate)truststore.getCertificate(alias), null));
			}	
		} catch(Exception e)
		{
			log.fatal("Can't initialize the list of trusted " +
					"CAs for web registration module!", e);
		}
	}

	@Override
	public Set<TrustAnchor> getTrustAnchors()
	{
		return taSet;
	}
}
